/*
**	LED-LC circuit
*/
#include "x.c"
#define W 500	// expand upto 1200 to see behavior
#define H 500
#define y1 60
#define y2 300
#define y3 (H-36)

//
// diode characteristic
//
#define Vd(i) (((i)>0)?log(i)/2.75+.3:0)	// 1A --> 0.3V (Why 2.75 (not 10)?)
#define Id(v) (((v)>0)?exp(10*((v)-.3)):0)	// 0.3V --> 1A

//
// simulation tic(unitoftime) and L(inductance or coil) and C(capacitance or condenser)
//
#define T 0.25	//[micro second] simulation tic(unitoftime)
#define L 5	//[micro Henry]  L(inductance or coil)
#define C 50	//[micro Farad]  C(capacitance or condenser)
#define Lpd 0	//use Loop Diode or not

void xdp(int a, int b) { XDrawPoint(dpy,w1,gc,a,b); }
void xdl(int a, int b, int c, int d) { XDrawLine(dpy,w1,gc,a,b,c,d); }
void mark(int i, int y, int v){
	if(i%4==0)xdp(i,y); if(i%100==0) xdl(i,y-5,i,y+5); xdp(i,y-v);
}

void led_lc() {
	int i,on;
	double vi, vc, vl, il, ic, io;
	on= io= vi= vc= ic= il= 0;

	for(i= 0; i< W; i++){
		if(i%100==0)on= 1 - on;

		if(on) { vi= 12; vl= vi - vc; }
		else if(Lpd) { vl= -Vd(il); vi= vl + vc; }
		      else   { vi= -Vd(il); vl= vi - vc; }

		il+= vl*T/L; if(il<0)il= 0;	// Why clipping is needed?

		if(Lpd && !on) ic= 0;
		else ic= il;

		io= exp(10*vc - 100);
		vc+= (ic - io)*T/C;
		mark(i, y1, 4*vi); mark(i, y2,16*vc); mark(i, y3, 3*ic);
	}
}

int main(argc,argv) char **argv; {
	iwt("LED-LC circuit ",W,H);
	led_lc();
	XFlush(dpy);
	printf("period=%6.3f[micro sec]\nPush Return key\n",2*M_PI*sqrt(L*C));
	getchar();
}
